#!/etc/Tivoli/bin/perl

####################################

&main();

####################################

sub main
{
  local($SomeIRLocation);


  $SomeIRLocation = &IR'GetIRLocation();
  if (length($SomeIRLocation) > 0)
  {
    if (&IR'CheckForIRLocks($SomeIRLocation))
    {
      print STDOUT "Unable to continue, SIS locks in place.\n";
    }
    else
    {
      # Update IND Files

      print STDOUT "\n";
      print STDOUT "Updating IND files for OS/2 Gateway support.\n";
      print STDOUT "\n";

      # Software Distribution 3.6 Gateway
      print STDOUT "  Software Distribution 3.6 Gateway...\n";
      &IR'RemoveComponentsFromINDFile("$SomeIRLocation/Products/CourierGw-3.6-TME_10_Software_Distribution_Gateway,_Version_3.6/COUR_LCF.IND",
                                      "BIN", "ALIDB", "DB");

      # Inventory 3.6 Gateway
      &IR'RemoveComponentsFromINDFile("$SomeIRLocation/Products/InventoryGW-3.6-TME_10_Inventory_Gateway,_Version_3.6/LCF.IND",
                                      "BIN");
      print STDOUT "  Inventory 3.6 Gateway...\n";

      # Security Management 3.6. Gateway
      &IR'RemoveComponentsFromINDFile("$SomeIRLocation/Products/SECMGTGW-3.6-TME_10_Security_Management_for_Gateways_3.6/SECMGTGW.IND",
                                      "BIN", "ALIDB");
      print STDOUT "  Security Management 3.6 Gateway...\n";

      print STDOUT "\n";

      print STDOUT "Successfully updated IND files for OS/2 Gateway support.\n";


      # Zap IR

      &IR'DeleteSAVFiles($SomeIRLocation);

      print STDOUT "\n";
      print STDOUT "Successfully zapped SIS IR.\n";
      print STDOUT "\n";
    }
  }
  else
  {
    print STDOUT "Unable to locate SIS IR.\n";
  }
}

####################################

package FileCalls;

####################################

sub FileCalls'MakeDirIfNecessary
{
  local($SomeDir) = @_;


  if (-e $SomeDir)
  {
    if (-d $SomeDir)
    {
      ;
    }
    else
    {

      die "Not a directory: \"$SomeDir\"\n";
    }
  }
  else
  {
    &FileCalls'MakeDir($SomeDir);
  }
}

####################################

sub FileCalls'MakeDir
{
  local($SomeDir) = @_;
  local($ReturnCode) = 0;


  $ReturnCode = system("mkdir \"$SomeDir\"");

  $ReturnCode = ( ! $ReturnCode);

  if ($ReturnCode)
  {
    $ReturnCode = system("chmod 777 \"$SomeDir\"");

    $ReturnCode = ( ! $ReturnCode);

    if ( ! $ReturnCode)
    {
      die "Unable to create directory: \"$SomeDir\".\n";
    }
  }
  else
  {
    die "Unable to create directory: \"$SomeDir\".\n";
  }
}

####################################

sub FileCalls'MoveFile
{
  local($SomeSource, $SomeDest, $SomeRelativeFile) = @_;
  local($SomeSourceFile, $SomeDestFile) = @_;
  local($ReturnCode) = 0;


  $ReturnCode = system("mv \"$SomeSourceFile\" \"$SomeDestFile\"");

  return ( ! $ReturnCode);
}

####################################

sub FileCalls'CopyFile
{
  local($SomeSourceFile, $SomeDestFile, $SomeRelativeFile) = @_;
  local($ReturnCode) = 0;


  $ReturnCode = system("cp \"$SomeSourceFile\" \"$SomeDestFile\"");

  if ( ! $ReturnCode)
  {
    $ReturnCode = system("chmod 666 \"$SomeDestFile\"");
  }

  return ( ! $ReturnCode);
}

####################################

sub FileCalls'SoftLinkFile
{
  local($SomeSourceFile, $SomeDestFile, $SomeRelativeFile) = @_;
  local($RelativeFile);
  local($ReturnCode) = 0;


  $RelativeFile = &FileCalls'MakeRelative($SomeSourceFile, $SomeDestFile);

  $ReturnCode = symlink($RelativeFile, $SomeDestFile);

  return ( ! $ReturnCode);
}

####################################

sub FileCalls'DeleteFile
{
  local($SomeFile) = @_;


  $ReturnCode = unlink($SomeFile);

  return ( ! $ReturnCode);
}

####################################

sub FileCalls'ParentDir
{
  local($SomeFile) = @_;
  local($bAbsolute) = 0;
  local($ParentDir);


  if (length($SomeFile) > 0)
  {
    if ($SomeFile[0] == '/')
    {
      $bAbsolute = 1;
    }

    @PathList = split("/", $SomeFile);

    # Remove "" if necessary
    if ($bAbsolute)
    {
      shift(@PathList);
    }

    # Remove first entry
    pop(@PathList);

    if ($bAbsolute)
    {
      $ParentDir = "/" . join("/", @PathList);
    }
    else
    {
      if ($#PathList < 0)
      {
        $ParentDir = ".";
      }
      else
      {
        $ParentDir = join("/", @PathList);
      }
    }
  }

  return $ParentDir;
}

####################################

sub FileCalls'BaseName
{
  local($SomeFilename) = @_;


  $SomeFilename =~ /([^\/]+)$/;
  $BaseName = $1;

  return $BaseName;
}

####################################

sub FileCalls'MakeRelative
{
  local($SomeSourceFile, $SomeDestFile) = @_;
  local(@Source, @Dest);
  local($NumDestComponents);
  local($bDone);
  local($SourceComponent, $DestComponent);
  local($RelativeFile);


  @Source = split("/", $SomeSourceFile);
  @Dest   = split("/", $SomeDestFile);

  # Absorb first "" component
  $SourceComponent = shift(@Source);
  $DestComponent   = shift(@Dest);

  $NumDestComponents = $#Dest;

  $bDone = 0;
  while ( ! $bDone)
  {
    if ($SourceComponent = shift(@Source))
    {
      if ($DestComponent = shift(@Dest))
      {
        if ($SourceComponent ne $DestComponent)
        {
          unshift(@Source, $SourceComponent);
          unshift(@Dest, $DestComponent);

          $bDone = 1;
        }
      }
      else
      {
        unshift(@Source, $SourceComponent);

        $bDone = 1;
      }
    }
    else
    {
      $bDone = 1;
    }
  }

  # If there are common components, trim on ".." from the up list.
  # Otherwise, we'll need it to navigate to the correct place.
  if ($NumDestComponents != $#Dest)
  {
    $DestComponent = pop(@Dest);
  }

  while ($DestComponent = pop(@Dest))
  {
    if (length($RelativeFile) > 0)
    {
      $RelativeFile .= "/";
    }

    $RelativeFile .= "..";
  }

  while ($SourceComponent = shift(@Source))
  {
    $RelativeFile .= "/";

    $RelativeFile .= $SourceComponent;
  }

  return $RelativeFile;
}

####################################

sub FileCalls'GetDirHier
{
  local($Prefix, $SomeDir) = @_;
  local(@Directory, $File);
  local(@DirHier);


  opendir(DIRHIER, $SomeDir) || die "Couldn't open directory '$SomeDir'.";
  while ($File = readdir(DIRHIER))
  {
    if (($File ne ".") &&
        ($File ne ".."))
    {
      push(@Directory, $File);
    }
  }
  closedir(DIRHIER);

  foreach $File (@Directory)
  {
    push(@DirHier, "$Prefix$File");

    if ( -d "$SomeDir$File")
    {
      push(@DirHier, &FileCalls'GetDirHier("$Prefix$File/", "$SomeDir$File/"));
    }
  }

  return @DirHier;
}

####################################

sub FileCalls'GetRelativeDirHier
{
  local($SomeDir) = @_;


  if (length($SomeDir) > 0)
  {
    if (substr($SomeDir, $#SomeDir, 1) ne "/")
    {
      $SomeDir .= "/";  # Add slash, if necessary
    }
  }

  return &FileCalls'GetDirHier("", $SomeDir);
}

####################################

sub FileCalls'GetAbsoluteDirHier
{
  local($SomeDir) = @_;


  if (length($SomeDir) > 0)
  {
    if (substr($SomeDir, $#SomeDir, 1) ne "/")
    {
      $SomeDir .= "/";  # Add slash, if necessary
    }
  }

  return &FileCalls'GetDirHier($SomeDir, $SomeDir);
}

####################################

package IR;

$IRLocation = "";

####################################

sub IR'GetIRLocation
{
  if (length($IRLocation) == 0)
  {
    local($SomeDBDIR) = $ENV{"DBDIR"};
    local($SomeIRLocation);
    local($AltIRLocation);


    ($SomeDBDIR) || die("You must first source your Tivoli environment.");

    open(IRLOCFILE, "<$SomeDBDIR/ir.loc") ||
        die("No 'ir.loc' file found in the database directory\n  '$SomeDBDIR'");

    ($SomeIRLocation = <IRLOCFILE>) ||
        die("No IR location defined in\n  '$SomeDBDIR/ir.loc'");

    chop($SomeIRLocation);

    close(IRLOCFILE);

    open(IRLOCFILE, "wlsinst -av |");
    while ($Line = <IRLOCFILE>)
    {
      if ($Line =~ /^ +IRLOC=(.*)$/)
      {
        $AltIRLocation = $1;
      }
    }
    close(IRLOCFILE);

    if ((length($AltIRLocation) > 0) &&
        ($SomeIRLocation ne $AltIRLocation))
    {
      die("IRLOC attribute and 'ir.loc' file contents mis-matched.  Run 'PointIR'.");
    }

    $IRLocation = $SomeIRLocation;
  }

  return $IRLocation;
}

####################################

sub IR'CheckForIRLocks
{
  local($SomeIRLocation) = @_;
  local(@Files);
  local($bIRLocked) = 0;


  if ( ! system("wlookup SIS"))
  {
    $bIRLocked = 1;
  }
  elsif (-e "$SomeIRLocation/ir.lck")
  {
    $bIRLocked = 1;
  }
  elsif (-e "$SomeIRLocation/TMR/Defaults/ir.lck")
  {
    $bIRLocked = 1;
  }
  else
  {
    @Files = &FileCalls'GetRelativeDirHier("$SomeIRLocation/TMR/Defaults/ULOCK");
    if ($#Files >= 0)
    {
      $bIRLocked = 1;
    }
  }

  return $bIRLocked;
}

####################################

sub IR'DeleteIRLocks
{
  local($SomeIRLocation) = @_;
  local($SomeFileOrDirectory);


  system("wregister -u SIS");

  if (-e "$SomeIRLocation/ir.lck")
  {
    &FileCalls'DeleteFile("$SomeIRLocation/ir.lck");
  }

  if (-e "$SomeIRLocation/TMR/Defaults/ir.lck")
  {
    &FileCalls'DeleteFile("$SomeIRLocation/TMR/Defaults/ir.lck");
  }

  @Files = &FileCalls'GetAbsoluteDirHier("$SomeIRLocation/TMR/Defaults/ULOCK");
  while ($SomeFileOrDirectory = shift(@Files))
  {
    &FileCalls'DeleteFile($SomeFileOrDirectory);
  }
}

####################################

sub IR'DeleteSAVFiles
{
  local($SomeIRLocation) = @_;
  local(@SomeIRDirHier, $File);
  local($Filename);


  @SomeIRDirHier = &FileCalls'GetAbsoluteDirHier($SomeIRLocation);

  foreach $File (@SomeIRDirHier)
  {
    $Filename = &FileCalls'BaseName($File);

    if (($Filename eq "product.sav") ||      # Delete 'product.sav' files
        ($Filename eq "tmr.sav") ||          # Delete 'tmr.sav' files
        ($Filename eq "minitmr.sav") ||      # Delete 'minitmr.sav' files
        ($Filename eq "minitmr.sav.BAK") ||
        ($Filename eq "miniprod.sav") ||     # Delete 'miniprod.sav' files
        ($Filename eq "miniprod.sav.BAK"))
    {
      &FileCalls'DeleteFile($File);
    }
  }
}

####################################

sub IR'INDFileToSISFile
{
  local($SomeIRDir, $SomeINDFile) = @_;
  local($Line);
  local($ProdTag) = "";
  local($LongName) = "";
  local($Revision) = "";
  local($bPatch) = 0;
  local($bFramework) = 0;
  local($SISName);
  local($SomeSISFile) = "";


  if (-e $SomeINDFile)
  {
    open(INDFILE, "<$SomeINDFile");
    while ($Line = <INDFILE>)
    {
      if ($Line =~ /^([^:]+):description:([^:]+)/)
      {
        $ProdTag = $1;
        $LongName = $2;
      }
      elsif ($Line =~ /^[^:]+:revison:([^:]+)/)
      {
        $Revision = $1;
      }
      elsif ($Line =~ /^[^:]+:patch_for:/)
      {
        $bPatch = 1;
      }
    }
    close(INDFILE);


    if ($ProdTag eq "TMF")
    {
      $bFramework = 1;
    }


    # Modify the description as necessary

    $Description =~ s/\s/_/g;
    $Description =~ s/\//-/g;


    # Create directory name itself

    $SISName .= "$ProdTag";

    if (length($Revision) > 0)
    {
      $SISName .= "-$Revision";
    }

    if ($bFramework)
    {
      $SomeSISFile = "$SomeIRDir/Framework/$SISName";
    }
    else
    {
      # Search for existing directory

      if (-e "$SomeIRDir/Framework/$SISName")
      {
        $SomeSISFile = "$SomeIRDir/Framework/$SISName";
      }
      else
      {
        $SISName .= "-Description";

        if (-e "$SomeIRDir/Products/$SISName")
        {
          $SomeSISFile = "$SomeIRDir/Products/$SISName";
        }
        elsif (-e "$SomeIRDir/Patches/$SISName")
        {
          $SomeSISFile = "$SomeIRDir/Patches/$SISName";
        }
        else
        {
          if ($bPatch)
          {
            $SomeSISFile = "$SomeIRDir/Patches/$SISName";
          }
          else
          {
            $SomeSISFile = "$SomeIRDir/Products/$SISName";
          }
        }
      }
    }

    $SomeSISFile .= &FileCalls'BaseName($SomeINDFile);
  }

  return $SomeSISFile;
}

####################################

####################################

sub IR'RemoveComponentsFromINDFile
{
  local($SomeINDFile, @Components) = @_;
  local(@INDData);
  local($SomeParentDir);
  local($SomeComponent);
  local(@FilesToRemove);
  local($FileNumber, $File);
  

  if ((-e $SomeINDFile) &&
      (-f $SomeINDFile) &&
      ($SomeINDFile =~ /\.IND$/))
  {
    open(INDFILE, "<$SomeINDFile");
    @INDData = <INDFILE>;
    close(INDFILE);

    $SomeParentDir = &FileCalls'ParentDir($SomeINDFile);

    foreach $SomeComponent (@Components)
    {
      &IR'FilterComponent($SomeComponent);  # INDData, FilesToRemove direct access

      foreach $FileNumber (@FilesToRemove)
      {
        # Remove File Packet
        $File = $SomeParentDir . "/FILE" . $FileNumber . ".PKT";
        &FileCalls'DeleteFile($File);

        # Remove CFG File
        $File = $SomeParentDir . "/CFG/FILE" . $FileNumber . ".CFG";
        &FileCalls'DeleteFile($File);
      }
    }

    open(INDFILE, ">$SomeINDFile");
    print INDFILE @INDData;
    close(INDFILE);
  }
}

####################################

sub IR'FilterComponent
{
  local($Filter) = @_;
  local($Line);
  local(@FilteredINDData);


  # Reset FileToRemove to 0 entries

  @FilesToRemove = ();

  foreach $Line (@INDData)
  {
    # Remove component lines
    # i.e.
    # CourierGw:id:BIN:...
    # CourierGw:fp:BIN:aix4-r1::7:23

    if ($Line =~ /^[^:]+:id:$Filter:/)
    {
      ;
    }
    elsif ($Line =~ /^[^:]+:fp:$Filter:.*:([^:]+)\n$/)
    {
      push(@FilesToRemove, $1);
    }
    else
    {
      push(@FilteredINDData, $Line);
    }
  }

  @INDData = @FilteredINDData;
}

####################################

